local MAJOR_VERSION, MINOR_VERSION = "1.0", string.match( "$Revision: 3 $", "%d+" ) or "dev"
local L = LibStub( "AceLocale-3.0" ):GetLocale( "NotaBene" )

NotaBene = LibStub( "AceAddon-3.0" ):NewAddon( "NotaBene", "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0" )
NotaBene.L = L

local defaults = {
	profile = {
		modules = {
			["*"] = true,
		},
	},
}

NotaBene:SetDefaultModuleState( false )

NotaBene.engines = {}

function NotaBene:OnInitialize()
	self.db = LibStub( "AceDB-3.0" ):New( "NotaBeneDB", defaults, "Default" )
	
	self.db.RegisterCallback( self, "OnNewProfile", "HandleProfileChanges" )
	self.db.RegisterCallback( self, "OnProfileReset", "HandleProfileChanges" )
	self.db.RegisterCallback( self, "OnProfileChanged", "HandleProfileChanges" )
	self.db.RegisterCallback( self, "OnProfileCopied", "HandleProfileChanges" )
	
	self:InitializeOptions()
	
	self:RegisterEvent( "PLAYER_REGEN_ENABLED" )
	self:RegisterEvent( "PLAYER_REGEN_DISABLED" )
	
	self:Print( L["Version %s.%s loaded. Type |cff00ffff/nb|r to configure."]:format( MAJOR_VERSION, MINOR_VERSION ) )
end

function NotaBene:OnEnable()
	self.inCombat = InCombatLockdown()
	
	self:HandleProfileChanges( "OnProfileChanged" )
end

function NotaBene:RegisterProfileDefaults( key, data )
	defaults.profile.key = data
end

function NotaBene:HandleProfileChanges( event, ... )
	-- Enabling/disabling modules
	
	for moduleName, module in self:IterateModules() do
		if self.db.profile.modules[moduleName] then
			if not module:IsEnabled() then
				module:Enable()
			end
			
			if type( module.HandleProfileChanges ) == "function" then
				module:HandleProfileChanges( event, ... )
			end
		else
			if module:IsEnabled() then
				module:Disable()
			end
		end
	end
end

function NotaBene:PLAYER_REGEN_DISABLED()
	self.inCombat = true
	
	for k, v in self:IterateModules() do
		if v:IsEnabled() and type( v.OnEnterCombat ) == "function" then
			v:OnEnterCombat()
		end
	end
end

function NotaBene:PLAYER_REGEN_ENABLED()
	self.inCombat = false
	
	for k, v in self:IterateModules() do
		if v:IsEnabled() and type( v.OnLeaveCombat ) == "function" then
			v:OnLeaveCombat()
		end
	end
end

local formatters = {
	time = {
		s = function( v ) return v end,
		S = function( v ) return v end,
		m = function( v ) return math.floor( v/60 ) end,
		M = function( v ) return math.floor( v/60 ) end,
		h = function( v ) return math.floor( v/3600 ) end,
		H = function( v ) return math.floor( v/3600 ) end,
	},
	text = function( v, fmt )
		return (fmt and #fmt > 0) and string.format( "|c%s%s|r", fmt, v ) or v
	end,
}

function NotaBene:FormatMessage( message, args )
	if not args then return message end
	
	message = string.gsub( message, "(%%([%a][%a%d]*)\:?([%a%d]*))", function( full, tag, fmt )
		tag = string.upper( tag )
		
		local arg = args[string.upper( tag )]
		local value = arg.value
		
		if not arg then return full end
		
		local formatter = formatters[arg.type]
		
		if not formatter then return value end
		
		if type( formatter ) == "table" and fmt and #fmt > 0 then
			formatter = formatter[fmt]
			
			if type( formatter ) == "function" then
				value = formatter( value )
			end
		elseif type( formatter ) == "function" then
			value = formatter( value, fmt )
		end
		
		return value
	end )
	
	return message
end

NotaBene.ModulePrototype = {}
NotaBene.ModulePrototype.__index = NotaBene.ModulePrototype

NotaBene:SetDefaultModulePrototype( NotaBene.ModulePrototype )
